var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { toObject, objectAsArray } from './lib/index.js';
export function newModel(attributes, info) {
    let attributesArr = [];
    for (let key in attributes) {
        attributesArr.push(Object.assign({ name: key }, attributes[key]));
    }
    return new Model(info.tableName, attributesArr);
}
class Model {
    constructor(tableName, attributes) {
        this.tableName = tableName;
        this.attributes = attributes;
    }
    init(database) {
        this.db = database;
    }
    create(obj) {
        return __awaiter(this, void 0, void 0, function* () {
            this.generate(obj);
            yield this.checkDuplicated(obj);
            this.checkRequired(obj);
            // Only write attributes declared on the model
            let objToWrite = {};
            for (let key of Object.keys(obj))
                for (let attr of this.attributes)
                    if (attr.name === key)
                        objToWrite[key] = obj[key];
            this.getData().push(objToWrite);
            this.db.write();
        });
    }
    findAll() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getData();
        });
    }
    findOne(query) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getData().find(entry => {
                return this.runQuery(query, entry);
            });
        });
    }
    find(query) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getData().filter(entry => {
                return this.runQuery(query, entry);
            });
        });
    }
    update(query, obj) {
        return __awaiter(this, void 0, void 0, function* () {
            (yield this.find(query)).map((entry) => {
                for (let key of Object.keys(obj))
                    for (let attr of this.attributes)
                        if (attr.name === key)
                            entry[key] = obj[key];
            });
            this.db.write();
        });
    }
    remove(query) {
        return __awaiter(this, void 0, void 0, function* () {
            // Filter the data saving only the results that did not match the query
            let newData = this.getData().filter(entry => {
                // If matched the query return false
                return !this.runQuery(query, entry);
            });
            this.write(newData);
        });
    }
    runQuery(query, entry) {
        let queries = objectAsArray(query);
        let entries = objectAsArray(entry);
        // Filter wich queries are met
        let queriesMet = queries.filter(query => {
            // Loop the entry to query
            // If the result length is one, then the query were met
            let matched = entries.filter(entry => query.name === entry.name
                && query.value === entry.value);
            return matched.length === 1;
        });
        // If all queries were met the length will be the same
        return queriesMet.length === queries.length;
    }
    // Get the data of the attribute with the name of the model tableName	
    getData() {
        if (!this.db.data)
            this.db.data = {};
        // Loop into data to check if it has the tableName attribute
        for (let key in this.db.data)
            if (key === this.tableName)
                return this.db.data[key];
        // If not found, create it		
        // Use the tableName as a PropertyKey
        let tableName = this.getTableNameAsPropertyKey();
        // Create an attribute inside data, wich name is the value of tableName
        // and set it's value as an empty array
        this.write([]);
        return this.db.data[tableName];
    }
    getTableNameAsPropertyKey() {
        // Table name is a key of data
        // And it's value is the table name
        let tableName = this.tableName;
        return tableName;
    }
    write(value) {
        return __awaiter(this, void 0, void 0, function* () {
            // Use the tableName as a PropertyKey
            let tableName = this.getTableNameAsPropertyKey();
            let data = this.db.data || {};
            data[tableName] = value;
            this.db.data = data;
            this.db.write();
        });
    }
    checkDuplicated(obj) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let attr of this.attributes)
                if (attr.unique
                    && attr.unique === true
                    && (yield this.findOne(toObject(attr.name, obj[attr.name]))))
                    throw new Error('Duplicated');
        });
    }
    checkRequired(obj) {
        for (let attr of this.attributes)
            if (attr.required
                && attr.required === true
                && !obj[attr.name])
                throw new Error('Required');
    }
    generate(obj) {
        for (let attr of this.attributes)
            if (attr.generate)
                obj[attr.name] = attr.generate();
    }
}
//# sourceMappingURL=Model.js.map